;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "real-auto-save" "26.1.0"
  "Automatically save your buffers/files at regular intervals."
  '((emacs "24.4"))
  :url "https://github.com/ChillarAnand/real-auto-save"
  :commit "b74a9fd408196c9333b465852c76525df0371ad1"
  :revdesc "b74a9fd40819"
  :authors '(("Chaoji Li" . "lichaojiATgmailDOTcom")
             ("Anand Reddy Pandikunta" . "anand21nandaATgmailDOTcom"))
  :maintainers '(("Chaoji Li" . "lichaojiATgmailDOTcom")
                 ("Anand Reddy Pandikunta" . "anand21nandaATgmailDOTcom")))
